// -*- C++ -*-
#include <ept/test.h>
#include <ept/core/list.h>
#include <ept/core/xapian.h>

// TODO temporary, for building test database
#include <ept/apt/apt.h>
#include <ept/textsearch/textsearch.h>
#include <ept/textsearch/maint/path.h>

using namespace ept;
using namespace core;

namespace ept {
namespace textsearch {
extern size_t max_index;
}
}

struct XapianTestEnvironment : AptTestEnvironment {
    textsearch::Path::OverrideIndexDir oid;
    textsearch::TextSearch textsearch;
    apt::Apt apt;
    XapianTestEnvironment()
        : oid( TEST_ENV_DIR "xapian" )
    {
        ept::textsearch::max_index = 1000;
        textsearch.rebuildIfNeeded(apt);
    }
};

struct TestXapian : XapianTestEnvironment {
    xapian::Source xap;
    Xapian::Enquire enq;

    TestXapian() :
        enq( xap.db() )
    {
        Xapian::Query q = xap.makeORQuery( "sgml tool" );
        enq.set_query( q );
    }

    Test enquire() {
	Xapian::MSet matches = enq.get_mset(0, 100);
        assert( matches.size() > 0 );
        assert_eq( matches.begin().get_document().get_data(), "psgml" );
    }

    Test listCount() {
        xapian::List l( enq );
	Xapian::MSet matches = enq.get_mset(0, 6000);
        assert_eq( list::count( l ), matches.size() );
    }

    template< typename List >
    void checkXapianList( List l, Xapian::MSet m ) {
        Xapian::MSet::const_iterator i = m.begin();
        while (! l.empty() ) {
            assert_eq( l.token().package(), i.get_document().get_data() );
            l = l.tail();
            ++ i;
        }
    }

    Test list() {
        xapian::List l( enq );
	Xapian::MSet matches = enq.get_mset(0, 6000);
        checkXapianList( l, matches );
    }

    Test query() {
        assert_eq( xap.query( "sgml tool" ).token().package(), "psgml" );
    }

};
